using System;
using Lazy;

namespace Lazy.Example
{

	public class BaseClass {

		public virtual int LazyMethod () {
			return 1;
		}

		public virtual float LazyMethod2 (bool b, byte bt, char c, short s, int i, long l, float f, double d, decimal m) {
			return 0;
		}
	}

	/// <summary>
	/// Descripcin breve de LazyClass.
	/// </summary>
	public class LazyClass: BaseClass
	{
		private LazyObjectModel model;

		public LazyClass()
		{
			model = LazyFactory.GetObjectModel( this.GetType().BaseType ); 
		}

		// MSIL tests


		private float SampleMSILnoParams () {
			return (float) model.InvokeMethod(this, 3, new object[]{});
		}

		private object SampleMSILObject () {
			return model.InvokeMethod(this, 3, new object[]{});
		}

		private object SampleMSILwithParam (int a) {

			object[] parameters = new object[1];

			parameters[0] = a;

			return model.InvokeMethod(this, 3, parameters);
		}

		private object SampleMSILwithParamObject (object a) {

			object[] parameters = new object[1];

			parameters[0] = a;

			return model.InvokeMethod(this, 3, parameters);
		}

		private object SampleMSILwithParams (int a, int b) {

			object[] parameters = new object[1];

			parameters[0] = a;
			parameters[1] = b;

			return model.InvokeMethod(this, 3, parameters);
		}

		private object SampleMSILwithParams (int a, int b, int c) {

			object[] parameters = new object[1];

			parameters[0] = a;
			parameters[1] = b;
			parameters[2] = c;

			return model.InvokeMethod(this, 3, parameters);
		}


		public override int LazyMethod () {

			int alternative = model.LazyMethodIndex(this,3);

			if (alternative!=0) {
				return (int) model.InvokeMethodImplementation(this, 3, alternative, new object[]{});
			} else {
				return base.LazyMethod();
			}
		}

		public override float LazyMethod2(bool b, byte bt, char c, short s, int i, long l, float f, double d, decimal m) {

			int alternative = model.LazyMethodIndex(this,3);

			if (alternative!=0) {
				return (float) model.InvokeMethodImplementation(this, 3, alternative, new object[]{});
			} else {
				return base.LazyMethod2 (b, bt, c, s, i, l, f, d, m);
			}			
		}



		// Example

		public override String ToString () {
			return model.ToString();
		}

	}
}
